/* valainterfaceregisterfunction.vala
 *
 * Copyright (C) 2006-2007  Jürg Billeter, Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>


#define VALA_TYPE_TYPEREGISTER_FUNCTION (vala_typeregister_function_get_type ())
#define VALA_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunction))
#define VALA_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))
#define VALA_IS_TYPEREGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_IS_TYPEREGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_TYPEREGISTER_FUNCTION))
#define VALA_TYPEREGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_TYPEREGISTER_FUNCTION, ValaTypeRegisterFunctionClass))

typedef struct _ValaTypeRegisterFunction ValaTypeRegisterFunction;
typedef struct _ValaTypeRegisterFunctionClass ValaTypeRegisterFunctionClass;
typedef struct _ValaTypeRegisterFunctionPrivate ValaTypeRegisterFunctionPrivate;

#define VALA_TYPE_INTERFACE_REGISTER_FUNCTION (vala_interface_register_function_get_type ())
#define VALA_INTERFACE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunction))
#define VALA_INTERFACE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunctionClass))
#define VALA_IS_INTERFACE_REGISTER_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_INTERFACE_REGISTER_FUNCTION))
#define VALA_IS_INTERFACE_REGISTER_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_INTERFACE_REGISTER_FUNCTION))
#define VALA_INTERFACE_REGISTER_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunctionClass))

typedef struct _ValaInterfaceRegisterFunction ValaInterfaceRegisterFunction;
typedef struct _ValaInterfaceRegisterFunctionClass ValaInterfaceRegisterFunctionClass;
typedef struct _ValaInterfaceRegisterFunctionPrivate ValaInterfaceRegisterFunctionPrivate;

/**
 * C function to register a type at runtime.
 */
struct _ValaTypeRegisterFunction {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaTypeRegisterFunctionPrivate * priv;
};

struct _ValaTypeRegisterFunctionClass {
	GTypeClass parent_class;
	void (*finalize) (ValaTypeRegisterFunction *self);
	ValaTypeSymbol* (*get_type_declaration) (ValaTypeRegisterFunction* self);
	char* (*get_type_struct_name) (ValaTypeRegisterFunction* self);
	char* (*get_base_init_func_name) (ValaTypeRegisterFunction* self);
	char* (*get_class_finalize_func_name) (ValaTypeRegisterFunction* self);
	char* (*get_base_finalize_func_name) (ValaTypeRegisterFunction* self);
	char* (*get_class_init_func_name) (ValaTypeRegisterFunction* self);
	char* (*get_instance_struct_size) (ValaTypeRegisterFunction* self);
	char* (*get_instance_init_func_name) (ValaTypeRegisterFunction* self);
	char* (*get_parent_type_name) (ValaTypeRegisterFunction* self);
	char* (*get_gtype_value_table_init_function_name) (ValaTypeRegisterFunction* self);
	char* (*get_gtype_value_table_peek_pointer_function_name) (ValaTypeRegisterFunction* self);
	char* (*get_gtype_value_table_free_function_name) (ValaTypeRegisterFunction* self);
	char* (*get_gtype_value_table_copy_function_name) (ValaTypeRegisterFunction* self);
	char* (*get_gtype_value_table_lcopy_value_function_name) (ValaTypeRegisterFunction* self);
	char* (*get_gtype_value_table_collect_value_function_name) (ValaTypeRegisterFunction* self);
	char* (*get_type_flags) (ValaTypeRegisterFunction* self);
	ValaCCodeFragment* (*get_type_interface_init_declaration) (ValaTypeRegisterFunction* self);
	ValaCCodeFragment* (*get_type_interface_init_statements) (ValaTypeRegisterFunction* self);
	ValaSymbolAccessibility (*get_accessibility) (ValaTypeRegisterFunction* self);
};

/**
 * C function to register an interface at runtime.
 */
struct _ValaInterfaceRegisterFunction {
	ValaTypeRegisterFunction parent_instance;
	ValaInterfaceRegisterFunctionPrivate * priv;
};

struct _ValaInterfaceRegisterFunctionClass {
	ValaTypeRegisterFunctionClass parent_class;
};

struct _ValaInterfaceRegisterFunctionPrivate {
	ValaInterface* _interface_reference;
};


static gpointer vala_interface_register_function_parent_class = NULL;

gpointer vala_typeregister_function_ref (gpointer instance);
void vala_typeregister_function_unref (gpointer instance);
GParamSpec* vala_param_spec_typeregister_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_typeregister_function (GValue* value, gpointer v_object);
gpointer vala_value_get_typeregister_function (const GValue* value);
GType vala_typeregister_function_get_type (void);
GType vala_interface_register_function_get_type (void);
#define VALA_INTERFACE_REGISTER_FUNCTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_INTERFACE_REGISTER_FUNCTION, ValaInterfaceRegisterFunctionPrivate))
enum  {
	VALA_INTERFACE_REGISTER_FUNCTION_DUMMY_PROPERTY
};
void vala_interface_register_function_set_interface_reference (ValaInterfaceRegisterFunction* self, ValaInterface* value);
void vala_typeregister_function_set_context (ValaTypeRegisterFunction* self, ValaCodeContext* value);
ValaInterfaceRegisterFunction* vala_interface_register_function_new (ValaInterface* iface, ValaCodeContext* context);
ValaInterfaceRegisterFunction* vala_interface_register_function_construct (GType object_type, ValaInterface* iface, ValaCodeContext* context);
ValaInterfaceRegisterFunction* vala_interface_register_function_new (ValaInterface* iface, ValaCodeContext* context);
ValaInterface* vala_interface_register_function_get_interface_reference (ValaInterfaceRegisterFunction* self);
static ValaTypeSymbol* vala_interface_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_base_init_func_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_class_finalize_func_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_base_finalize_func_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base);
static char* vala_interface_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base);
static ValaSymbolAccessibility vala_interface_register_function_real_get_accessibility (ValaTypeRegisterFunction* base);
static ValaCCodeFragment* vala_interface_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base);
static void vala_interface_register_function_finalize (ValaTypeRegisterFunction* obj);



ValaInterfaceRegisterFunction* vala_interface_register_function_construct (GType object_type, ValaInterface* iface, ValaCodeContext* context) {
	ValaInterfaceRegisterFunction* self;
	g_return_val_if_fail (iface != NULL, NULL);
	g_return_val_if_fail (context != NULL, NULL);
	self = (ValaInterfaceRegisterFunction*) g_type_create_instance (object_type);
	vala_interface_register_function_set_interface_reference (self, iface);
	vala_typeregister_function_set_context ((ValaTypeRegisterFunction*) self, context);
	return self;
}


ValaInterfaceRegisterFunction* vala_interface_register_function_new (ValaInterface* iface, ValaCodeContext* context) {
	return vala_interface_register_function_construct (VALA_TYPE_INTERFACE_REGISTER_FUNCTION, iface, context);
}


static ValaTypeSymbol* vala_interface_register_function_real_get_type_declaration (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	ValaTypeSymbol* result;
	ValaTypeSymbol* _tmp0_;
	self = (ValaInterfaceRegisterFunction*) base;
	_tmp0_ = NULL;
	result = (_tmp0_ = (ValaTypeSymbol*) self->priv->_interface_reference, (_tmp0_ == NULL) ? NULL : vala_code_node_ref (_tmp0_));
	return result;
}


static char* vala_interface_register_function_real_get_type_struct_name (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = vala_interface_get_type_cname (self->priv->_interface_reference);
	return result;
}


static char* vala_interface_register_function_real_get_base_init_func_name (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	char* _tmp0_;
	char* _tmp1_;
	self = (ValaInterfaceRegisterFunction*) base;
	_tmp0_ = NULL;
	_tmp1_ = NULL;
	result = (_tmp1_ = g_strdup_printf ("%s_base_init", _tmp0_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) self->priv->_interface_reference, NULL)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
	return result;
}


static char* vala_interface_register_function_real_get_class_finalize_func_name (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = g_strdup ("NULL");
	return result;
}


static char* vala_interface_register_function_real_get_base_finalize_func_name (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = g_strdup ("NULL");
	return result;
}


static char* vala_interface_register_function_real_get_class_init_func_name (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = g_strdup ("NULL");
	return result;
}


static char* vala_interface_register_function_real_get_instance_struct_size (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = g_strdup ("0");
	return result;
}


static char* vala_interface_register_function_real_get_instance_init_func_name (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = g_strdup ("NULL");
	return result;
}


static char* vala_interface_register_function_real_get_parent_type_name (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	char* result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = g_strdup ("G_TYPE_INTERFACE");
	return result;
}


static ValaSymbolAccessibility vala_interface_register_function_real_get_accessibility (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	ValaSymbolAccessibility result;
	self = (ValaInterfaceRegisterFunction*) base;
	result = vala_symbol_get_access ((ValaSymbol*) self->priv->_interface_reference);
	return result;
}


static ValaCCodeFragment* vala_interface_register_function_real_get_type_interface_init_statements (ValaTypeRegisterFunction* base) {
	ValaInterfaceRegisterFunction * self;
	ValaCCodeFragment* result;
	ValaCCodeFragment* frag;
	self = (ValaInterfaceRegisterFunction*) base;
	frag = vala_ccode_fragment_new ();
	/* register all prerequisites */
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _prereq_ref_it;
		/* register all prerequisites */
		_tmp0_ = NULL;
		_tmp1_ = NULL;
		_prereq_ref_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_interface_get_prerequisites (self->priv->_interface_reference))), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (gee_collection_object_unref (_tmp0_), NULL)), _tmp1_);
		/* register all prerequisites */
		while (TRUE) {
			ValaDataType* prereq_ref;
			ValaTypeSymbol* _tmp2_;
			ValaTypeSymbol* prereq;
			ValaCCodeIdentifier* _tmp3_;
			ValaCCodeFunctionCall* _tmp4_;
			ValaCCodeFunctionCall* func;
			ValaCCodeIdentifier* _tmp7_;
			char* _tmp6_;
			char* _tmp5_;
			ValaCCodeIdentifier* _tmp9_;
			char* _tmp8_;
			ValaCCodeExpressionStatement* _tmp10_;
			/* register all prerequisites */
			if (!gee_iterator_next (_prereq_ref_it)) {
				/* register all prerequisites */
				break;
			}
			/* register all prerequisites */
			prereq_ref = (ValaDataType*) gee_iterator_get (_prereq_ref_it);
			_tmp2_ = NULL;
			prereq = (_tmp2_ = vala_data_type_get_data_type (prereq_ref), (_tmp2_ == NULL) ? NULL : vala_code_node_ref (_tmp2_));
			_tmp3_ = NULL;
			_tmp4_ = NULL;
			func = (_tmp4_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new ("g_type_interface_add_prerequisite"))), (_tmp3_ == NULL) ? NULL : (_tmp3_ = (vala_ccode_node_unref (_tmp3_), NULL)), _tmp4_);
			_tmp7_ = NULL;
			_tmp6_ = NULL;
			_tmp5_ = NULL;
			vala_ccode_function_call_add_argument (func, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = g_strdup_printf ("%s_type_id", _tmp5_ = vala_symbol_get_lower_case_cname ((ValaSymbol*) self->priv->_interface_reference, NULL)))));
			(_tmp7_ == NULL) ? NULL : (_tmp7_ = (vala_ccode_node_unref (_tmp7_), NULL));
			_tmp6_ = (g_free (_tmp6_), NULL);
			_tmp5_ = (g_free (_tmp5_), NULL);
			_tmp9_ = NULL;
			_tmp8_ = NULL;
			vala_ccode_function_call_add_argument (func, (ValaCCodeExpression*) (_tmp9_ = vala_ccode_identifier_new (_tmp8_ = vala_typesymbol_get_type_id (prereq))));
			(_tmp9_ == NULL) ? NULL : (_tmp9_ = (vala_ccode_node_unref (_tmp9_), NULL));
			_tmp8_ = (g_free (_tmp8_), NULL);
			_tmp10_ = NULL;
			vala_ccode_fragment_append (frag, (ValaCCodeNode*) (_tmp10_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) func)));
			(_tmp10_ == NULL) ? NULL : (_tmp10_ = (vala_ccode_node_unref (_tmp10_), NULL));
			(prereq_ref == NULL) ? NULL : (prereq_ref = (vala_code_node_unref (prereq_ref), NULL));
			(prereq == NULL) ? NULL : (prereq = (vala_code_node_unref (prereq), NULL));
			(func == NULL) ? NULL : (func = (vala_ccode_node_unref (func), NULL));
		}
		(_prereq_ref_it == NULL) ? NULL : (_prereq_ref_it = (gee_collection_object_unref (_prereq_ref_it), NULL));
	}
	result = frag;
	return result;
}


ValaInterface* vala_interface_register_function_get_interface_reference (ValaInterfaceRegisterFunction* self) {
	ValaInterface* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_interface_reference;
	return result;
}


void vala_interface_register_function_set_interface_reference (ValaInterfaceRegisterFunction* self, ValaInterface* value) {
	g_return_if_fail (self != NULL);
	self->priv->_interface_reference = value;
}


static void vala_interface_register_function_class_init (ValaInterfaceRegisterFunctionClass * klass) {
	vala_interface_register_function_parent_class = g_type_class_peek_parent (klass);
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->finalize = vala_interface_register_function_finalize;
	g_type_class_add_private (klass, sizeof (ValaInterfaceRegisterFunctionPrivate));
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_type_declaration = vala_interface_register_function_real_get_type_declaration;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_type_struct_name = vala_interface_register_function_real_get_type_struct_name;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_base_init_func_name = vala_interface_register_function_real_get_base_init_func_name;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_class_finalize_func_name = vala_interface_register_function_real_get_class_finalize_func_name;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_base_finalize_func_name = vala_interface_register_function_real_get_base_finalize_func_name;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_class_init_func_name = vala_interface_register_function_real_get_class_init_func_name;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_instance_struct_size = vala_interface_register_function_real_get_instance_struct_size;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_instance_init_func_name = vala_interface_register_function_real_get_instance_init_func_name;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_parent_type_name = vala_interface_register_function_real_get_parent_type_name;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_accessibility = vala_interface_register_function_real_get_accessibility;
	VALA_TYPEREGISTER_FUNCTION_CLASS (klass)->get_type_interface_init_statements = vala_interface_register_function_real_get_type_interface_init_statements;
}


static void vala_interface_register_function_instance_init (ValaInterfaceRegisterFunction * self) {
	self->priv = VALA_INTERFACE_REGISTER_FUNCTION_GET_PRIVATE (self);
}


static void vala_interface_register_function_finalize (ValaTypeRegisterFunction* obj) {
	ValaInterfaceRegisterFunction * self;
	self = VALA_INTERFACE_REGISTER_FUNCTION (obj);
	VALA_TYPEREGISTER_FUNCTION_CLASS (vala_interface_register_function_parent_class)->finalize (obj);
}


GType vala_interface_register_function_get_type (void) {
	static GType vala_interface_register_function_type_id = 0;
	if (vala_interface_register_function_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaInterfaceRegisterFunctionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_interface_register_function_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaInterfaceRegisterFunction), 0, (GInstanceInitFunc) vala_interface_register_function_instance_init, NULL };
		vala_interface_register_function_type_id = g_type_register_static (VALA_TYPE_TYPEREGISTER_FUNCTION, "ValaInterfaceRegisterFunction", &g_define_type_info, 0);
	}
	return vala_interface_register_function_type_id;
}




