/* valabasicblock.vala
 *
 * Copyright (C) 2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_BASIC_BLOCK (vala_basic_block_get_type ())
#define VALA_BASIC_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_BASIC_BLOCK, ValaBasicBlock))
#define VALA_BASIC_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockClass))
#define VALA_IS_BASIC_BLOCK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_BASIC_BLOCK))
#define VALA_IS_BASIC_BLOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_BASIC_BLOCK))
#define VALA_BASIC_BLOCK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockClass))

typedef struct _ValaBasicBlock ValaBasicBlock;
typedef struct _ValaBasicBlockClass ValaBasicBlockClass;
typedef struct _ValaBasicBlockPrivate ValaBasicBlockPrivate;

#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))

typedef struct _ValaCodeNode ValaCodeNode;
typedef struct _ValaCodeNodeClass ValaCodeNodeClass;

#define VALA_TYPE_PHI_FUNCTION (vala_phi_function_get_type ())
#define VALA_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunction))
#define VALA_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))
#define VALA_IS_PHI_FUNCTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PHI_FUNCTION))
#define VALA_IS_PHI_FUNCTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PHI_FUNCTION))
#define VALA_PHI_FUNCTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PHI_FUNCTION, ValaPhiFunctionClass))

typedef struct _ValaPhiFunction ValaPhiFunction;
typedef struct _ValaPhiFunctionClass ValaPhiFunctionClass;
typedef struct _ValaParamSpecBasicBlock ValaParamSpecBasicBlock;

/**
 * Represents a basic block, i.e. a straight-line piece of code without any
 * jumps or jump targets.
 */
struct _ValaBasicBlock {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaBasicBlockPrivate * priv;
};

struct _ValaBasicBlockClass {
	GTypeClass parent_class;
	void (*finalize) (ValaBasicBlock *self);
};

struct _ValaBasicBlockPrivate {
	GeeList* nodes;
	GeeList* predecessors;
	GeeList* successors;
	ValaBasicBlock* _parent;
	GeeList* children;
	GeeSet* df;
	GeeSet* phi_functions;
};

struct _ValaParamSpecBasicBlock {
	GParamSpec parent_instance;
};


static gpointer vala_basic_block_parent_class = NULL;

gpointer vala_basic_block_ref (gpointer instance);
void vala_basic_block_unref (gpointer instance);
GParamSpec* vala_param_spec_basic_block (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_basic_block (GValue* value, gpointer v_object);
gpointer vala_value_get_basic_block (const GValue* value);
GType vala_basic_block_get_type (void);
gpointer vala_code_node_ref (gpointer instance);
void vala_code_node_unref (gpointer instance);
GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_code_node (GValue* value, gpointer v_object);
gpointer vala_value_get_code_node (const GValue* value);
GType vala_code_node_get_type (void);
gpointer vala_phi_function_ref (gpointer instance);
void vala_phi_function_unref (gpointer instance);
GParamSpec* vala_param_spec_phi_function (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_phi_function (GValue* value, gpointer v_object);
gpointer vala_value_get_phi_function (const GValue* value);
GType vala_phi_function_get_type (void);
#define VALA_BASIC_BLOCK_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_BASIC_BLOCK, ValaBasicBlockPrivate))
enum  {
	VALA_BASIC_BLOCK_DUMMY_PROPERTY
};
ValaBasicBlock* vala_basic_block_new (void);
ValaBasicBlock* vala_basic_block_construct (GType object_type);
ValaBasicBlock* vala_basic_block_new (void);
ValaBasicBlock* vala_basic_block_new_entry (void);
ValaBasicBlock* vala_basic_block_construct_entry (GType object_type);
ValaBasicBlock* vala_basic_block_new_entry (void);
ValaBasicBlock* vala_basic_block_new_exit (void);
ValaBasicBlock* vala_basic_block_construct_exit (GType object_type);
ValaBasicBlock* vala_basic_block_new_exit (void);
void vala_basic_block_add_node (ValaBasicBlock* self, ValaCodeNode* node);
GeeList* vala_basic_block_get_nodes (ValaBasicBlock* self);
void vala_basic_block_connect (ValaBasicBlock* self, ValaBasicBlock* target);
GeeList* vala_basic_block_get_predecessors (ValaBasicBlock* self);
GeeList* vala_basic_block_get_successors (ValaBasicBlock* self);
static void vala_basic_block_set_parent (ValaBasicBlock* self, ValaBasicBlock* value);
void vala_basic_block_add_child (ValaBasicBlock* self, ValaBasicBlock* block);
GeeList* vala_basic_block_get_children (ValaBasicBlock* self);
void vala_basic_block_add_dominator_frontier (ValaBasicBlock* self, ValaBasicBlock* block);
GeeSet* vala_basic_block_get_dominator_frontier (ValaBasicBlock* self);
void vala_basic_block_add_phi_function (ValaBasicBlock* self, ValaPhiFunction* phi);
GeeSet* vala_basic_block_get_phi_functions (ValaBasicBlock* self);
ValaBasicBlock* vala_basic_block_get_parent (ValaBasicBlock* self);
static void vala_basic_block_finalize (ValaBasicBlock* obj);



ValaBasicBlock* vala_basic_block_construct (GType object_type) {
	ValaBasicBlock* self;
	self = (ValaBasicBlock*) g_type_create_instance (object_type);
	return self;
}


ValaBasicBlock* vala_basic_block_new (void) {
	return vala_basic_block_construct (VALA_TYPE_BASIC_BLOCK);
}


ValaBasicBlock* vala_basic_block_construct_entry (GType object_type) {
	ValaBasicBlock* self;
	self = (ValaBasicBlock*) g_type_create_instance (object_type);
	return self;
}


ValaBasicBlock* vala_basic_block_new_entry (void) {
	return vala_basic_block_construct_entry (VALA_TYPE_BASIC_BLOCK);
}


ValaBasicBlock* vala_basic_block_construct_exit (GType object_type) {
	ValaBasicBlock* self;
	self = (ValaBasicBlock*) g_type_create_instance (object_type);
	return self;
}


ValaBasicBlock* vala_basic_block_new_exit (void) {
	return vala_basic_block_construct_exit (VALA_TYPE_BASIC_BLOCK);
}


void vala_basic_block_add_node (ValaBasicBlock* self, ValaCodeNode* node) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (node != NULL);
	gee_collection_add ((GeeCollection*) self->priv->nodes, node);
}


GeeList* vala_basic_block_get_nodes (ValaBasicBlock* self) {
	GeeList* result;
	GeeList* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	result = (_tmp0_ = self->priv->nodes, (_tmp0_ == NULL) ? NULL : gee_collection_object_ref (_tmp0_));
	return result;
}


void vala_basic_block_connect (ValaBasicBlock* self, ValaBasicBlock* target) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (target != NULL);
	if (!gee_collection_contains ((GeeCollection*) self->priv->successors, target)) {
		gee_collection_add ((GeeCollection*) self->priv->successors, target);
	}
	if (!gee_collection_contains ((GeeCollection*) target->priv->predecessors, self)) {
		gee_collection_add ((GeeCollection*) target->priv->predecessors, self);
	}
}


GeeList* vala_basic_block_get_predecessors (ValaBasicBlock* self) {
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GeeList*) gee_read_only_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, self->priv->predecessors);
	return result;
}


GeeList* vala_basic_block_get_successors (ValaBasicBlock* self) {
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GeeList*) gee_read_only_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, self->priv->successors);
	return result;
}


void vala_basic_block_add_child (ValaBasicBlock* self, ValaBasicBlock* block) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	gee_collection_add ((GeeCollection*) self->priv->children, block);
	vala_basic_block_set_parent (block, self);
}


GeeList* vala_basic_block_get_children (ValaBasicBlock* self) {
	GeeList* result;
	GeeList* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	result = (_tmp0_ = self->priv->children, (_tmp0_ == NULL) ? NULL : gee_collection_object_ref (_tmp0_));
	return result;
}


void vala_basic_block_add_dominator_frontier (ValaBasicBlock* self, ValaBasicBlock* block) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (block != NULL);
	gee_collection_add ((GeeCollection*) self->priv->df, block);
}


GeeSet* vala_basic_block_get_dominator_frontier (ValaBasicBlock* self) {
	GeeSet* result;
	GeeSet* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	result = (_tmp0_ = self->priv->df, (_tmp0_ == NULL) ? NULL : gee_collection_object_ref (_tmp0_));
	return result;
}


void vala_basic_block_add_phi_function (ValaBasicBlock* self, ValaPhiFunction* phi) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (phi != NULL);
	gee_collection_add ((GeeCollection*) self->priv->phi_functions, phi);
}


GeeSet* vala_basic_block_get_phi_functions (ValaBasicBlock* self) {
	GeeSet* result;
	GeeSet* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	result = (_tmp0_ = self->priv->phi_functions, (_tmp0_ == NULL) ? NULL : gee_collection_object_ref (_tmp0_));
	return result;
}


ValaBasicBlock* vala_basic_block_get_parent (ValaBasicBlock* self) {
	ValaBasicBlock* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_parent;
	return result;
}


static void vala_basic_block_set_parent (ValaBasicBlock* self, ValaBasicBlock* value) {
	ValaBasicBlock* _tmp1_;
	ValaBasicBlock* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_parent = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : vala_basic_block_ref (_tmp0_)), (self->priv->_parent == NULL) ? NULL : (self->priv->_parent = (vala_basic_block_unref (self->priv->_parent), NULL)), _tmp1_);
}


static void vala_value_basic_block_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_basic_block_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_basic_block_unref (value->data[0].v_pointer);
	}
}


static void vala_value_basic_block_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_basic_block_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_basic_block_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_basic_block_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaBasicBlock* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_basic_block_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_basic_block_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaBasicBlock** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_basic_block_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_basic_block (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecBasicBlock* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_BASIC_BLOCK), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_basic_block (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_BASIC_BLOCK), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_basic_block (GValue* value, gpointer v_object) {
	ValaBasicBlock* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_BASIC_BLOCK));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_BASIC_BLOCK));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_basic_block_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_basic_block_unref (old);
	}
}


static void vala_basic_block_class_init (ValaBasicBlockClass * klass) {
	vala_basic_block_parent_class = g_type_class_peek_parent (klass);
	VALA_BASIC_BLOCK_CLASS (klass)->finalize = vala_basic_block_finalize;
	g_type_class_add_private (klass, sizeof (ValaBasicBlockPrivate));
}


static void vala_basic_block_instance_init (ValaBasicBlock * self) {
	self->priv = VALA_BASIC_BLOCK_GET_PRIVATE (self);
	self->priv->nodes = (GeeList*) gee_array_list_new (VALA_TYPE_CODE_NODE, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal);
	self->priv->predecessors = (GeeList*) gee_array_list_new (VALA_TYPE_BASIC_BLOCK, NULL, NULL, g_direct_equal);
	self->priv->successors = (GeeList*) gee_array_list_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_equal);
	self->priv->children = (GeeList*) gee_array_list_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_equal);
	self->priv->df = (GeeSet*) gee_hash_set_new (VALA_TYPE_BASIC_BLOCK, (GBoxedCopyFunc) vala_basic_block_ref, vala_basic_block_unref, g_direct_hash, g_direct_equal);
	self->priv->phi_functions = (GeeSet*) gee_hash_set_new (VALA_TYPE_PHI_FUNCTION, (GBoxedCopyFunc) vala_phi_function_ref, vala_phi_function_unref, g_direct_hash, g_direct_equal);
	self->ref_count = 1;
}


static void vala_basic_block_finalize (ValaBasicBlock* obj) {
	ValaBasicBlock * self;
	self = VALA_BASIC_BLOCK (obj);
	(self->priv->nodes == NULL) ? NULL : (self->priv->nodes = (gee_collection_object_unref (self->priv->nodes), NULL));
	(self->priv->predecessors == NULL) ? NULL : (self->priv->predecessors = (gee_collection_object_unref (self->priv->predecessors), NULL));
	(self->priv->successors == NULL) ? NULL : (self->priv->successors = (gee_collection_object_unref (self->priv->successors), NULL));
	(self->priv->_parent == NULL) ? NULL : (self->priv->_parent = (vala_basic_block_unref (self->priv->_parent), NULL));
	(self->priv->children == NULL) ? NULL : (self->priv->children = (gee_collection_object_unref (self->priv->children), NULL));
	(self->priv->df == NULL) ? NULL : (self->priv->df = (gee_collection_object_unref (self->priv->df), NULL));
	(self->priv->phi_functions == NULL) ? NULL : (self->priv->phi_functions = (gee_collection_object_unref (self->priv->phi_functions), NULL));
}


GType vala_basic_block_get_type (void) {
	static GType vala_basic_block_type_id = 0;
	if (vala_basic_block_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_basic_block_init, vala_value_basic_block_free_value, vala_value_basic_block_copy_value, vala_value_basic_block_peek_pointer, "p", vala_value_basic_block_collect_value, "p", vala_value_basic_block_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaBasicBlockClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_basic_block_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaBasicBlock), 0, (GInstanceInitFunc) vala_basic_block_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_basic_block_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaBasicBlock", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vala_basic_block_type_id;
}


gpointer vala_basic_block_ref (gpointer instance) {
	ValaBasicBlock* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_basic_block_unref (gpointer instance) {
	ValaBasicBlock* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_BASIC_BLOCK_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




